<?php
/*
  ZFS kernel tune extension for NAS4Free/FreeNAS Legacy.
  Copyright (C) 2010-2012 Daisuke Aoyama <aoyama@peach.ne.jp>.
  All rights reserved.
*/
require("auth.inc");
require("guiconfig.inc");
if (version_compare(PHP_VERSION, '5.4.0') < 0) {
   require_once("./zfskerntune-compat.php");
}

$pgtitle = array(gettext("Extensions"), gettext("ZFS kernel tune"));

$pconfig['zfstune'] = '';

$pconfig['disable_prefetch'] = true;
$pconfig['zil_disable'] = false;
// ZFS v13 default
//$pconfig['txgtimeout'] = 30;
//$pconfig['vdevmaxpending'] = 35;
//$pconfig['vdevminpending'] = 4;

function zfskerntune_get_spa() {
	$spa = @exec("/sbin/sysctl -q -n vfs.zfs.version.spa");
	if ($spa == '') $spa = 6;
	return $spa;
}

$zfs_defaults = array();
$spa = zfskerntune_get_spa();
if ($spa < 15) {
	// ZFS v13 default
	$zfs_defaults['txgtimeout'] = 30;
	$zfs_defaults['vdevmaxpending'] = 35;
	$zfs_defaults['vdevminpending'] = 4;
} else if ($spa >= 15) {
	// ZFS v15 default
	$zfs_defaults['txgtimeout'] = 5;
	$zfs_defaults['vdevmaxpending'] = 10;
	$zfs_defaults['vdevminpending'] = 4;
	if ($spa >= 28) {
		// ZFS v28 default
		$zfs_defaults['write_limit_override'] = 0;
	}
}

// set defaults
$pconfig['txgtimeout'] = $zfs_defaults['txgtimeout'];
$pconfig['vdevmaxpending'] = $zfs_defaults['vdevmaxpending'];
$pconfig['vdevminpending'] = $zfs_defaults['vdevminpending'];
$pconfig['write_limit_override'] = $zfs_defaults['write_limit_override'];

// load current values
$info = zfskerntune_get_zfskernel_info();
if (!empty($info)) {
	if (array_key_exists('kmem', $info)) {
	}
	if (array_key_exists('arcmin', $info)) {
	}
	if (array_key_exists('arcmax', $info)) {
	}
	if (array_key_exists('disable_prefetch', $info)) {
		$pconfig['disable_prefetch'] = $info['disable_prefetch'];
	}
	if (array_key_exists('zil_disable', $info)) {
		$pconfig['zil_disable'] = $info['zil_disable'];
	}
	// ZFS v13 values
	if (array_key_exists('txgtimeout', $info)) {
		$pconfig['txgtimeout'] = $info['txgtimeout'];
	}
	if (array_key_exists('vdevmaxpending', $info)) {
		$pconfig['vdevmaxpending'] = $info['vdevmaxpending'];
	}
	if (array_key_exists('vdevminpending', $info)) {
		$pconfig['vdevminpending'] = $info['vdevminpending'];
	}
	// ZFS v28 values
	if (array_key_exists('write_limit_override', $info)) {
		$pconfig['write_limit_override'] = $info['write_limit_override'];
	}
}

if (!empty($_POST)) {
	unset($input_errors);
	unset($errormsg);
	$pconfig = $_POST;

	$zfstune = $_POST['zfstune'];
	$a_zfs_preset = zfskerntune_preset();
	$preset = $a_zfs_preset[$zfstune];
	$spa = zfskerntune_get_spa();

	if ($zfstune == '' || $zfstune == 'none') {
		$input_errors[] = gettext("no selection of memory size.");
	} else if ($zfstune == 'default') {
		$preset['kmem'] = '';
		$preset['arcmin'] = '';
		$preset['arcmax'] = '';
	} else {
		if (empty($preset) || !is_array($preset)) {
			$input_errors[] = gettext("no preset.");
		}
	}
	// default
/*
	if ($spa >= 13) {
		$preset['txgtimeout'] = 30;
		$preset['vdevmaxpending'] = 35;
		$preset['vdevminpending'] = 4;
	} else {
		$preset['txgtimeout'] = '';
		$preset['vdevmaxpending'] = '';
		$preset['vdevminpending'] = '';
	}
*/

	$userval = array();
	$userval['disable_prefetch'] = $pconfig['disable_prefetch'];
	if ($spa >= 28) {
		$userval['zil_disable'] = false;
	} else {
		$userval['zil_disable'] = $pconfig['zil_disable'];
	}
	if ($spa >= 13) {
		// Input validation.
		$reqdfields = explode(" ", "txgtimeout vdevmaxpending vdevminpending");
		$reqdfieldsn = array(gettext("txg.timeout"),
			gettext("Discovery vdev.max_pending"),
			gettext("Discovery vdev.min_pending"));
		$reqdfieldst = explode(" ", "numericint numericint numericint");

		if (version_compare(PHP_VERSION, '5.4.0') >= 0) {
			do_input_validation($_POST, $reqdfields, $reqdfieldsn, $input_errors);
			do_input_validation_type($_POST, $reqdfields, $reqdfieldsn, $reqdfieldst, $input_errors);
		} else {
			do_input_validation_53($_POST, $reqdfields, $reqdfieldsn, $input_errors);
			do_input_validation_type_53($_POST, $reqdfields, $reqdfieldsn, $reqdfieldst, $input_errors);
		}

		if (empty($input_errors)) {
			// set user values
			$userval['txgtimeout'] = $pconfig['txgtimeout'];
			$userval['vdevmaxpending'] = $pconfig['vdevmaxpending'];
			$userval['vdevminpending'] = $pconfig['vdevminpending'];
		}
	}
	if ($spa >= 28) {
		// Input validation.
		$reqdfields = explode(" ", "write_limit_override");
		$reqdfieldsn = array(gettext("write_limit_override"));
		$reqdfieldst = explode(" ", "string");

		if (version_compare(PHP_VERSION, '5.4.0') >= 0) {
			do_input_validation($_POST, $reqdfields, $reqdfieldsn, $input_errors);
			do_input_validation_type($_POST, $reqdfields, $reqdfieldsn, $reqdfieldst, $input_errors);
		} else {
			do_input_validation_53($_POST, $reqdfields, $reqdfieldsn, $input_errors);
			do_input_validation_type_53($_POST, $reqdfields, $reqdfieldsn, $reqdfieldst, $input_errors);
		}

		if (empty($input_errors)) {
			// set user values
			$userval['write_limit_override'] = $pconfig['write_limit_override'];
		}
	}

	if (empty($input_errors)) {
		// read current and replace it
		if (($conf = zfskerntune_read_loader_conf()) === FALSE) {
			$input_errors[] = gettext("config read error.");
		} else {
			$confnew = zfskerntune_replace_values($conf, $preset, $userval);
		}
	}
	if (empty($input_errors)) {
		// write new values
		if (zfskerntune_write_loader_conf($confnew) === FALSE) {
			$input_errors[] = gettext("config write error.");
		} else {
			$savemsg = gettext("The setting was saved successfully.");
		}
	}
}

function zfskerntune_read_loader_conf() {
	global $g;
	$result = array();
	$file = "loader.conf";
	if ($g['platform'] != "full") {
		$dir = "{$g['cf_path']}/boot";
		$ro = 1;
	} else {
		$dir = "/boot";
		$ro = 0;
	}
	$confpath = "$dir/$file";
	if (!file_exists($confpath)) {
		return FALSE;
	}

	$fp = fopen($confpath, "r");
	if (!$fp) {
		return FALSE;
	}
	while (!feof($fp)) {
		$line = fgets($fp);
		$result[] = $line;
	}
	fclose($fp);
	return $result;
}

function zfskerntune_write_loader_conf($contents) {
	global $g;
	$file = "loader.conf";
	if ($g['platform'] != "full") {
		$dir = "{$g['cf_path']}/boot";
		$ro = 1;
	} else {
		$dir = "/boot";
		$ro = 0;
	}
	$confpath = "$dir/$file";
	if (!file_exists($confpath)) {
		return FALSE;
	}

	if ($ro) {
		config_lock();
		conf_mount_rw();
	}
	$fp = fopen("{$confpath}.new", "w");
	if (!$fp) {
		if ($ro) {
			conf_mount_ro();
			config_unlock();
		}
		return FALSE;
	}
	foreach ($contents as $line) {
		if (fputs($fp, $line) === FALSE) {
			if ($ro) {
				conf_mount_ro();
				config_unlock();
			}
			return FALSE;
		}
	}
	fclose($fp);
	if (chmod("{$confpath}.new", 0644) === FALSE) {
		if ($ro) {
			conf_mount_ro();
			config_unlock();
		}
		return FALSE;
	}
	if (file_exists("{$confpath}.old") && (unlink("{$confpath}.old") === FALSE)) {
		if ($ro) {
			conf_mount_ro();
			config_unlock();
		}
		return FALSE;
	}
	if (rename("{$confpath}", "{$confpath}.old") === FALSE) {
		if ($ro) {
			conf_mount_ro();
			config_unlock();
		}
		return FALSE;
	}
	if (rename("{$confpath}.new", "{$confpath}") === FALSE) {
		if ($ro) {
			conf_mount_ro();
			config_unlock();
		}
		return FALSE;
	}
	if (unlink("{$confpath}.old") === FALSE) {
		if ($ro) {
			conf_mount_ro();
			config_unlock();
		}
		return FALSE;
	}
	if ($ro) {
		conf_mount_ro();
		config_unlock();
	}
	return TURE;
}

function zfskerntune_replace_values($contents, $preset, $userval) {
	$result = array();
	$header = "# ZFS kernel tune";
	// remove old lines
	foreach ($contents as $line) {
		if (preg_match("/^#?vm\.kmem.*/", $line)
		    || preg_match("/^#?vfs\.zfs\..*/", $line)) {
			continue;
		}
		if (preg_match("/^{$header}/", $line)) {
			continue;
		}
		$result[] = $line;
	}
	// add new values
	if ($preset['kmem'] != '') {
		$result[] = $header."\n";
		$result[] = "vm.kmem_size=\"${preset['kmem']}\"\n";
		$result[] = "vfs.zfs.arc_min=\"${preset['arcmin']}\"\n";
		$result[] = "vfs.zfs.arc_max=\"${preset['arcmax']}\"\n";
		if ($userval['disable_prefetch']) {
			$result[] = "vfs.zfs.prefetch_disable=\"1\"\n";
		} else {
			$result[] = "vfs.zfs.prefetch_disable=\"0\"\n";
		}
		if ($userval['zil_disable']) {
			$result[] = "vfs.zfs.zil_disable=\"1\"\n";
		} else {
			$result[] = "vfs.zfs.zil_disable=\"0\"\n";
		}
		// ZFS v13 values
		if (isset($userval['txgtimeout'])) {
			$result[] = "vfs.zfs.txg.timeout=\"${userval['txgtimeout']}\"\n";
			$result[] = "vfs.zfs.vdev.max_pending=\"${userval['vdevmaxpending']}\"\n";
			$result[] = "vfs.zfs.vdev.min_pending=\"${userval['vdevminpending']}\"\n";
		}
		// ZFS v28 values
		if (isset($userval['write_limit_override'])) {
			$result[] = "vfs.zfs.write_limit_override=\"${userval['write_limit_override']}\"\n";
		}
	}
	return $result;
}

// ZFS kernel
function zfskerntune_get_zfskernel_info() {
	$zfskern = array();

	$kmem_size = @exec("/bin/kenv -q vm.kmem_size");
	if (!empty($kmem_size) && $keme_size != '') {
		$zfskern['kmem'] = $kmem_size;
	}
	$arc_min = @exec("/bin/kenv -q vfs.zfs.arc_min");
	if (!empty($arc_min) && $arc_min != '') {
		$zfskern['arcmin'] = $arc_min;
	}
	$arc_max = @exec("/bin/kenv -q vfs.zfs.arc_max");
	if (!empty($arc_max) && $arc_max != '') {
		$zfskern['arcmax'] = $arc_max;
	}
	$disable_prefetch = @exec("/bin/kenv -q vfs.zfs.prefetch_disable");
	if (!empty($disable_prefetch) && $disable_prefetch != '') {
		$zfskern['disable_prefetch'] = $disable_prefetch != 0 ? true : false;
	}
	$zil_disable = @exec("/bin/kenv -q vfs.zfs.zil_disable");
	if (!empty($zil_disable) && $zil_disable != '') {
		$zfskern['zil_disable'] = $zil_disable != 0 ? true : false;
	}
	// ZFS v13 values
	$txgtimeout = @exec("/bin/kenv -q vfs.zfs.txg.timeout");
	if (!empty($txgtimeout) && $txgtimeout != '') {
		$zfskern['txgtimeout'] = $txgtimeout;
	}
	$minpending = @exec("/bin/kenv -q vfs.zfs.vdev.min_pending");
	if (!empty($minpending) && $minpending != '') {
		$zfskern['vdevminpending'] = $minpending;
	}
	$maxpending = @exec("/bin/kenv -q vfs.zfs.vdev.max_pending");
	if (!empty($maxpending) && $maxpending != '') {
		$zfskern['vdevmaxpending'] = $maxpending;
	}
	// ZFS v28 values
	$write_limit_override = @exec("/bin/kenv -q vfs.zfs.write_limit_override");
	if (!empty($write_limit_override) && $write_limit_override != '') {
		$zfskern['write_limit_override'] = $write_limit_override;
	}
	return $zfskern;
}

// SMBIOS
function zfskerntune_get_smbios_info() {
	$smbios = array();
	$cmd = "/bin/kenv -q";
	unset($rawdata, $rc);
	mwexec2($cmd, $rawdata, $rc);
	if ($rc != 0) {
		return $smbios;
	}
	foreach ($rawdata as $line) {
		if (preg_match("/^smbios\.([^\.=]+)\.([^\.=]+)=\"(.*)\"/", $line, $match)) {
			$smbios[$match[1]][$match[2]] = $match[3];
		}
	}
	return $smbios;
}

// ZFS kernel tune values
function zfskerntune_preset() {
	$a_zfsv6_preset = array(
		"X32_512MB"  => array("kmem" =>  "340M", "arcmin" => "50M", "arcmax" => "50M"),
		"X32_1024MB" => array("kmem" =>  "512M", "arcmin" => "128M", "arcmax" => "128M"),
		"X32_1536MB" => array("kmem" => "1024M", "arcmin" => "256M", "arcmax" => "256M"),
		"X32_2048MB" => array("kmem" => "1400M", "arcmin" => "350M", "arcmax" => "350M"),
		"X64_2GB"    => array("kmem" => "1536M", "arcmin" => "384M", "arcmax" => "384M"),
		"X64_3GB"    => array("kmem" => "2048M", "arcmin" => "512M", "arcmax" => "512M"),
		"X64_4GB"    => array("kmem" => "2560M", "arcmin" => "640M", "arcmax" => "640M"),
		"X64_6GB"    => array("kmem" => "4096M", "arcmin" => "1024M", "arcmax" => "1024M"),
		"X64_8GB"    => array("kmem" => "6144M", "arcmin" => "1536M", "arcmax" => "1536M"),
		"X64_12GB"   => array("kmem" => "9216M", "arcmin" => "2304M", "arcmax" => "2304M"),
		"X64_16GB"   => array("kmem" => "12288M", "arcmin" => "3072M", "arcmax" => "3072M"),
		"X64_24GB"   => array("kmem" => "20480M", "arcmin" => "5120M", "arcmax" => "5120M"),
		"X64_32GB"   => array("kmem" => "28672M", "arcmin" => "7168M", "arcmax" => "7168M"),
		);
	$a_zfsv13_preset = array(
		"X32_512MB"  => array("kmem" =>  "340M", "arcmin" => "60M", "arcmax" => "60M"),
		"X32_1024MB" => array("kmem" =>  "512M", "arcmin" => "128M", "arcmax" => "128M"),
		"X32_1536MB" => array("kmem" => "1024M", "arcmin" => "256M", "arcmax" => "256M"),
		"X32_2048MB" => array("kmem" => "1400M", "arcmin" => "400M", "arcmax" => "400M"),
		"X64_2GB"    => array("kmem" => "1536M", "arcmin" => "512M", "arcmax" => "512M"),
		"X64_3GB"    => array("kmem" => "2048M", "arcmin" => "1024M", "arcmax" => "1024M"),
		"X64_4GB"    => array("kmem" => "2560M", "arcmin" => "1536M", "arcmax" => "1536M"),
		"X64_6GB"    => array("kmem" => "4608M", "arcmin" => "3072M", "arcmax" => "3072M"),
		"X64_8GB"    => array("kmem" => "6656M", "arcmin" => "5120M", "arcmax" => "5120M"),
		"X64_12GB"   => array("kmem" => "10752M", "arcmin" => "9216M", "arcmax" => "9216M"),
		"X64_16GB"   => array("kmem" => "14336M", "arcmin" => "12288M", "arcmax" => "12288M"),
		"X64_24GB"   => array("kmem" => "22528M", "arcmin" => "20480M", "arcmax" => "20480M"),
		"X64_32GB"   => array("kmem" => "30720M", "arcmin" => "28672M", "arcmax" => "28672M"),
		"X64_48GB"   => array("kmem" => "47104M", "arcmin" => "45056M", "arcmax" => "45056M"),
		"X64_64GB"   => array("kmem" => "62464M", "arcmin" => "59392M", "arcmax" => "59392M"),
		"X64_96GB"   => array("kmem" => "95232M", "arcmin" => "92160M", "arcmax" => "92160M"),
		"X64_128GB"   => array("kmem" => "128000M", "arcmin" => "124928M", "arcmax" => "124928M"),
		"X64_192GB"   => array("kmem" => "193536M", "arcmin" => "190464M", "arcmax" => "190464M"),
		"X64_256GB"   => array("kmem" => "259072M", "arcmin" => "256000M", "arcmax" => "256000M"),
		);

	$spa = zfskerntune_get_spa();

	$a_zfs_preset = array();
	if ($spa == 6) {
		// ZFS v6
		foreach ($a_zfsv6_preset as $key => $val) {
			$a_zfs_preset[$key] = $val;
		}
	} else {
		// ZFS v13 or later
		foreach ($a_zfsv13_preset as $key => $val) {
			$a_zfs_preset[$key] = $val;
		}
	}
	return $a_zfs_preset;
}

function zfskerntune_list() {
	global $g;
	$strfmt = gettext("Memory Size: %s");
	$spa = zfskerntune_get_spa();

	$a_zfsall_list = array(
		"none" => gettext("select memory size below"),
		"default" => sprintf(gettext("default %s (remove)"),
				     $spa == '' ? "ZFS v6" : "ZFS v".$spa),
		);
	$a_zfsx32_list = array(
		"X32_512MB"  => sprintf($strfmt, gettext("512MB")),
		"X32_1024MB" => sprintf($strfmt, gettext("1GB")),
		"X32_1536MB" => sprintf($strfmt, gettext("1.5GB")),
		"X32_2048MB" => sprintf($strfmt, gettext("2GB (or more)")),
		);
	$a_zfsx64_list = array(
		"X64_2GB"  => sprintf($strfmt, gettext("2GB")),
		"X64_3GB"  => sprintf($strfmt, gettext("3GB")),
		"X64_4GB"  => sprintf($strfmt, gettext("4GB")),
		"X64_6GB"  => sprintf($strfmt, gettext("6GB")),
		"X64_8GB"  => sprintf($strfmt, gettext("8GB")),
		"X64_12GB" => sprintf($strfmt, gettext("12GB")),
		"X64_16GB" => sprintf($strfmt, gettext("16GB")),
		"X64_24GB" => sprintf($strfmt, gettext("24GB")),
		"X64_32GB" => sprintf($strfmt, gettext("32GB")),
		);
	$a_zfsx64v13_list = array(
		"X64_48GB" => sprintf($strfmt, gettext("48GB")),
		"X64_64GB" => sprintf($strfmt, gettext("64GB")),
		"X64_96GB" => sprintf($strfmt, gettext("96GB")),
		"X64_128GB" => sprintf($strfmt, gettext("128GB")),
		"X64_192GB" => sprintf($strfmt, gettext("192GB")),
		"X64_256GB" => sprintf($strfmt, gettext("256GB")),
		);

	$a_zfs_list = array();
	foreach ($a_zfsall_list as $key => $val) {
		$a_zfs_list[$key] = $val;
	}
	foreach ($a_zfsx32_list as $key => $val) {
		$a_zfs_list[$key] = $val;
	}
	if ($g['arch'] == "amd64" || $g['arch'] == "x64") {
		unset($a_zfs_list["X32_2048MB"]);
		foreach ($a_zfsx64_list as $key => $val) {
			$a_zfs_list[$key] = $val;
		}
		if ($spa >= 13) {
			foreach ($a_zfsx64v13_list as $key => $val) {
				$a_zfs_list[$key] = $val;
			}
		}
	}
	return $a_zfs_list;
}
?>
<?php include("fbegin.inc");?>
<script type="text/javascript">//<![CDATA[
//]]>
</script>
<form action="zfskerntune.php" method="post" name="iform" id="iform">
<table width="100%" border="0" cellpadding="0" cellspacing="0">
<tr>
  <td class="tabcont">
    <?php if (!empty($errormsg)) print_error_box($errormsg);?>
    <?php if (!empty($input_errors)) print_input_errors($input_errors);?>
    <?php if (!empty($savemsg)) print_info_box($savemsg);?>
    <table width="100%" border="0" cellpadding="6" cellspacing="0">
    <?php html_titleline(gettext("SMBIOS information"));?>
    <?php
	$smbios = zfskerntune_get_smbios_info();
	// BIOS Information
	echo html_text("smbios_bios_vendor", "BIOS Vendor", htmlspecialchars($smbios['bios']['vendor']));
	echo html_text("smbios_bios_version", "BIOS Version", htmlspecialchars($smbios['bios']['version']));
	echo html_text("smbios_bios_reldate", "BIOS Release Date", htmlspecialchars($smbios['bios']['reldate']));
	// System Information
	echo html_text("smbios_system_maker", "System Maker", htmlspecialchars($smbios['system']['maker']));
	echo html_text("smbios_system_product", "System Product", htmlspecialchars($smbios['system']['product']));
	echo html_text("smbios_system_version", "System Version", htmlspecialchars($smbios['system']['version']));
	echo html_text("smbios_system_serial", "System Serial", htmlspecialchars($smbios['system']['serial']));
	echo html_text("smbios_system_uuid", "System UUID", htmlspecialchars($smbios['system']['uuid']));
	// Base Board
	echo html_text("smbios_planar_maker", "Planar Maker", htmlspecialchars($smbios['planar']['maker']));
	echo html_text("smbios_planar_product", "Planar Product", htmlspecialchars($smbios['planar']['product']));
	echo html_text("smbios_planar_version", "Planar Version", htmlspecialchars($smbios['planar']['version']));
	echo html_text("smbios_planar_serial", "Planar Serial", htmlspecialchars($smbios['planar']['serial']));
	// System Enclosure or Chassis
	echo html_text("smbios_chassis_maker", "Chassis Maker", htmlspecialchars($smbios['chassis']['maker']));
	echo html_text("smbios_chassis_version", "Chassis Version", htmlspecialchars($smbios['chassis']['version']));
	echo html_text("smbios_chassis_serial", "Chassis Serial", htmlspecialchars($smbios['chassis']['serial']));
	echo html_text("smbios_chassis_tag", "Chassis Tag", htmlspecialchars($smbios['chassis']['tag']));
    ?>
    <?php html_separator();?>
    <?php html_titleline(gettext("ZFS kernel tune"));?>
    <tr>
      <td width="22%" valign="top" class="vncell"><?=gettext("ZFS kernel tune");?></td>
      <td width="78%" class="vtable">
      <select name='zfstune' class='formfld' id='zfstune'>
      <?php
	$spa = zfskerntune_get_spa();
	$a_list = zfskerntune_list();
	foreach ($a_list as $key => $val) {
	    $sel = ($pconfig['zfstune'] === $key) ? " selected" : "";
	    echo "<option value='$key'$sel>".htmlspecialchars($val)."</option>\n";
	}
      ?>
      </select>
      <br /><span class='vexpl'><?php echo sprintf("%s<br />%s", gettext("Changes to this option will take effect after a reboot."), gettext("If you want to run a large application such as virtual machine, please select a less memory than a physical memory."));?></span>
      </td>
    </tr>
    <?php html_checkbox("disable_prefetch", gettext("disable prefetch"), $pconfig['disable_prefetch'] ? true : false, gettext("Disable prefetch"), "", false, "");?>
    <?php if ($spa >= 28): ?>
    <?php
	echo html_text("zil_disable", gettext("disable ZIL"), gettext("disable ZIL was changed into each setup as Sync."));
    ?>
    <?php else: ?>
    <?php html_checkbox("zil_disable", gettext("disable ZIL"), $pconfig['zil_disable'] ? true : false, gettext("Disable ZIL"), "", false, "");?>
    <?php endif; ?>
    <?php
	if ($spa >= 13):
    ?>
    <?php html_separator();?>
    <?php html_titleline(gettext("ZFS kernel tune (v13 or later only)"));?>
    <?php html_inputbox("txgtimeout", gettext("txg.timeout"), $pconfig['txgtimeout'], sprintf(gettext("(%d by default)"), $zfs_defaults['txgtimeout']), false, 15, false);?>
    <?php html_inputbox("vdevmaxpending", gettext("vdev.max_pending"), $pconfig['vdevmaxpending'], sprintf(gettext("(%d by default)"), $zfs_defaults['vdevmaxpending']), false, 15, false);?>
    <?php html_inputbox("vdevminpending", gettext("vdev.min_pending"), $pconfig['vdevminpending'], sprintf(gettext("(%d by default)"), $zfs_defaults['vdevminpending']), false, 15, false);?>
    <? endif; ?>
    <?php if ($spa >= 28): ?>
    <?php html_separator();?>
    <?php html_titleline(gettext("ZFS kernel tune (v28 or later only)"));?>
    <?php html_inputbox("write_limit_override", gettext("write_limit_override"), $pconfig['write_limit_override'], sprintf(gettext("(%d by default)"), $zfs_defaults['write_limit_override']), false, 15, false);?>
    <? endif; ?>
    </table>
    <div id="submit">
      <input name="Submit" type="submit" class="formbtn" value="<?=gettext("Save");?>" />
    </div>
    <div id="remarks">
      <?php html_remark("note", gettext("Note"), gettext("If you need custom values, please edit loader.conf."));?>
    </div>
  </td>
</tr>
</table>
<?php include("formend.inc");?>
</form>
<script type="text/javascript">//<![CDATA[
//]]>
</script>
<?php include("fend.inc");?>
